# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from . import admissionregistration as admissionregistration
from . import apiextensions as apiextensions
from . import apiregistration as apiregistration
from . import apiserverinternal as apiserverinternal
from . import apps as apps
from . import authentication as authentication
from . import authorization as authorization
from . import autoscaling as autoscaling
from . import batch as batch
from . import certificates as certificates
from . import coordination as coordination
from . import core as core
from . import discovery as discovery
from . import events as events
from . import flowcontrol as flowcontrol
from . import networking as networking
from . import node as node
from . import policy as policy
from . import rbac as rbac
from . import resource as resource
from . import scheduling as scheduling
from . import storage as storage
